"""
Snakefile for the paper:

Benchmarking methods for selecting marker genes in single-cell RNA sequencing
data.

Author: Jeffrey Pullin (2020/2021)
Affiliation: University of Melbourne

This snakefile is designed to be run by the run.sh script in this directory.
"""

import json
import re

configfile: "config.yaml"
R = config["R"]

sim_pars = glob_wildcards("config/sim_pars/{sim_par}.json").sim_par
method_pars = glob_wildcards("config/method_pars/{method_par}.json").method_par
all_data_ids = config["all_data_ids"]
general_sim_data_ids = config["general_sim_data_ids"]
time_sim_data_ids = config["time_sim_data_ids"]

# Split the simulation parameters into those used to assess speed and others. 
# Then we can run the speed simulations on only a subset of the dataset, as 
# the simulation parameters are (largely) irrelevant for speed trends. 
# TODO: Ultimately it would be better to have a more convinient system to 
# control which datasets are paired with which simulation parameters. Note 
# that in this system the dataset ids need to visible to snakemake, to ensure
# they are orchestrated through the system correctly. 
time_pattern = re.compile('^num_*')
time_sim_pars = [par for par in sim_pars if time_pattern.match(par)]
general_sim_pars = [par for par in sim_pars if not time_pattern.match(par)]

time_sim_data_combs = expand(
  "{sim_par}-{data_id}", 
  sim_par = time_sim_pars, 
  data_id = time_sim_data_ids
)

general_sim_data_combs = expand(
  "{sim_par}-{data_id}", 
  sim_par = general_sim_pars, 
  data_id = general_sim_data_ids
)

sim_data_combs = time_sim_data_combs + general_sim_data_combs 

data_dirs = expand(
  config["real_data_folder"] + "{data_id}.rds",
  data_id = all_data_ids
)

sim_dirs = [config["sim_data_folder"] + x + ".rds" for x in sim_data_combs]

sim_data_res_dirs = expand(
  config["sim_data_results_folder"] + "{sim_data_comb}-{method_par}.rds", 
  method_par = method_pars,
  sim_data_comb = sim_data_combs
)

print(sim_dirs)
print(sim_data_res_dirs)

mg_dirs = expand(
  config["sim_data_folder"] + "{mg_type}-{sim_data_comb}.rds",
  sim_par = sim_pars, 
  mg_type = config["mg_type"],
  sim_data_comb = sim_data_combs
)

real_data_res_dirs = expand(
  config["real_data_results_folder"] + "{data_id}-{method_par}.rds", 
  method_par = method_pars,
  data_id = all_data_ids
)

shell("module load snakemake/6.6.1")
shell("module load r/4.1.0")

rule all:
  input: sim_dirs, sim_data_res_dirs, real_data_res_dirs, mg_dirs, data_dirs

rule prepare_data: 
  priority: 102
  input: script = config["code_folder"] + "prep-{data_id}.R",
    raw_data = config["raw_data_folder"] + "{data_id}.rds"
  output: data = config["real_data_folder"] + "{data_id}.rds"
  log: config["logs_folder"] + "prep_data-{data_id}.Rout"
 	resources: 
	  mem_mb=lambda wildcards, attempt: attempt * 6000,
	  time_min=lambda wildcards, attempt: [30, 60, 120][attempt - 1]
	shell: '''R CMD BATCH --no-restore --no-save\
		"--args raw_data={input.raw_data} data={output.data}"\
		{input.script} {log}'''
		
rule run_method_real_data:
	priority: 101
	input:	script = config["code_folder"] + "run-method.R",
			data = config["real_data_folder"] + "{data_id}.rds",
			method_pars = config["method_pars_folder"] + "{method_id}.json",
			fun = lambda wc: config["code_folder"] + "run-" + wc.method_id.split('_')[0] + ".R"
	output: res = config["real_data_results_folder"] + "{data_id}-{method_id}.rds"
	log:	config["logs_folder"] + "run_method_real_data-{data_id}-{method_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 6000,
	  time_min=lambda wildcards, attempt: [20, 60, 600][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={input.data} fun={input.fun}\
		meth_pars={input.method_pars} res={output.res}"\
		{input.script} {log}'''
	
rule simulate_data:
  priority: 100
	input:  script = config["code_folder"] + "simulate-data.R",
			sim_pars = config["sim_pars_folder"] + "{sim_id}.json",
			data = config["real_data_folder"] + "{data_id}.rds"
	output: sim = config["sim_data_folder"] + "{sim_id}-{data_id}.rds"
	log:	config["logs_folder"] + "sim_data-{sim_id}-{data_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 6000,
	  time_min=lambda wildcards, attempt: [30, 60, 600][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={output.sim} data={input.data}\
		sim_pars={input.sim_pars} wcs={wildcards}"\
		{input.script} {log}'''

rule extract_marker_genes:
	input:  script = config["code_folder"] + "extract-marker-genes.R",
	    sim_data = config["sim_data_folder"] + "{sim_id}-{data_id}.rds",
	output: mgs = config["sim_data_folder"] + "{mg_type}-{sim_id}-{data_id}.rds"
	log:	config["logs_folder"] + "extract_mgs-{mg_type}-{sim_id}-{data_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 6000,
	  time_min=lambda wildcards, attempt: [30, 60, 600][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={input.sim_data} mg_type={wildcards.mg_type}\
		mgs={output.mgs}"\
		{input.script} {log}'''

rule run_method_simulated_data:
	priority: 99
	input:	script = config["code_folder"] + "run-method.R",
			sim_data = config["sim_data_folder"] + "{sim_id}-{data_id}.rds",
			method_pars = config["method_pars_folder"] + "{method_id}.json",
			fun = lambda wc: config["code_folder"] + "run-" + wc.method_id.split('_')[0] + ".R"
	output: res = config["sim_data_results_folder"] + "{sim_id}-{data_id}-{method_id}.rds"
	log:	config["logs_folder"] + "run_method_simulated_data-{sim_id}-{data_id}-{method_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 8000,
	  time_min=lambda wildcards, attempt: [20, 60, 720][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={input.sim_data} fun={input.fun}\
		meth_pars={input.method_pars} res={output.res}"\
		{input.script} {log}'''
